local function getInventoryItemLevels()
	local averageIlvl, equippedIlvl, pvpIlvl = GetAverageItemLevel();
	local multiplyer = 100;
	averageIlvl = floor(multiplyer * averageIlvl)/multiplyer;
	equippedIlvl = floor(multiplyer * equippedIlvl)/multiplyer;
	pvpIlvl = floor(multiplyer * pvpIlvl) / multiplyer;
	local iLvls = {total = averageIlvl, equipped = equippedIlvl, pvp = pvpIlvl};
	return iLvls;
end

local function GetItemLevel()
	local iLvls = getInventoryItemLevels();
	if (UnitInBattleground("player")) then
		return iLvls.pvp;
	else
		return iLvls.equipped;
	end
end

local function CheckStats()
	local addonList = {"DejaCharacterStats"}; -- List of addons, that already changes the char stats window
	local disableCharStats = false;
	
	for _, v in pairs(addonList) do
		local _, _, _, enabled = GetAddOnInfo(v);
		if (enabled) then
			disableCharStats = true;
			break;
		end
	end
	
	return disableCharStats;
end

DLU.StatFrames = {
	["ItemLevelFrame"] = {
		frame = CharacterStatsPane.ItemLevelFrame,
		updateFunc = function(statFrame, unit)
			ilvl = GetItemLevel();
			PaperDollFrame_SetLabelAndText(statFrame, STAT_AVERAGE_ITEM_LEVEL, ilvl, false, ilvl);
			statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE .. format("Item Level: %s", ilvl) .. FONT_COLOR_CODE_CLOSE;
		end
	}
}

local hideStats = CheckStats();
if (not hideStats) then
	for k, v in pairs(DLU.StatFrames) do
		local index = 0;
		if (not v.frame) then
			local category = PAPERDOLL_STATCATEGORIES[1];
			local categoryFrame = category.categoryFrame;

			for _, v in pairs(CharacterStatsPane[categoryFrame]) do
				if (v) then
					index = index+1;
				end
			end

			v.frame = CreateFrame("Frame", nil, CharacterStatsPane[categoryFrame], "CharacterStatFrameTemplate");
			local height = v.frame:GetHeight();
			local xCord = -(index*height);
			v.frame:SetPoint("BOTTOMLEFT", 4, xCord);
			index = index+1;
		end
	end
end

function DLU.showStats(unit)
	if (not hideStats) then
		for _, v in pairs(DLU.StatFrames) do
			if (v.updateFunc) then
				if (v.frame) then
					v.updateFunc(v.frame, unit);
				end
			end
		end
	end
end