local addonName, addonData = ...;

-- Variables
local f = CreateFrame("Frame");

local MOUNTTYPES = {
    GROUND = "ground",
    FLYING = "flying",
    DRAGONRIDING = "dragonriding"
}

-- Important local functions
local function CanDragonRide()
    if (IsSpellKnown(376777) and IsUsableSpell(368896)) then
        return true;
    end

    return false;
end

local function KnowsFlyingSkill()
    return IsSpellKnown(90265) or IsSpellKnown(34091) or IsSpellKnown(34090);
end

local function KnowsRidingSkill()
    return IsSpellKnown(90265) or IsSpellKnown(34091) or IsSpellKnown(34090) or IsSpellKnown(33388);
end

local function IsInInstance(...)
    local currentID = select(8, GetInstanceInfo());
    for i = 1, select('#', ...) do
        local id = select(i, ...);
        if currentID == id then
            return true
        end
    end
    return false
end

local function CanFly()
    if (not KnowsFlyingSkill()) then
        return false;
    end

    local instanceID = select(8, GetInstanceInfo());

    if (IsInInstance(1642, 1643, 1718) and not IsSpellKnown(278833)) then
        return false;
    end

    if (IsInInstance(2222) and not C_QuestLog.IsQuestFlaggedCompleted(63893) and not C_QuestLog.IsQuestFlaggedCompleted(65539)) then
        return false;
    end

    if (C_Scenario.IsInScenario()) then
        local scenarioType = select(10, C_Scenario.GetInfo())
        if (scenarioType == LE_SCENARIO_TYPE_WARFRONT) then
            return false
        end
    end

    return IsFlyableArea();
end

local function CanRide()
    if (not KnowsRidingSkill()) then
        return false;
    end

    return true;
end

local function IsUseable(mountId)
    local isUseable = C_MountJournal.GetMountUsabilityByID(mountId, true);
    return isUseable;
end

-- Global function
function DLU.MountUp()
    if (not IsMounted()) then
        if (not InCombatLockdown()) then
            if (DLUCharacterSettings.FavoriteMounts.DragonRiding ~= nil and CanDragonRide()) then
                C_MountJournal.SummonByID(DLUCharacterSettings.FavoriteMounts.DragonRiding);
            elseif (DLUCharacterSettings.FavoriteMounts.Flying and CanFly() and IsUseable(DLUCharacterSettings.FavoriteMounts.Flying)) then
                C_MountJournal.SummonByID(DLUCharacterSettings.FavoriteMounts.Flying);
            elseif (CanRide() and IsUseable(DLUCharacterSettings.FavoriteMounts.Ground)) then
                C_MountJournal.SummonByID(DLUCharacterSettings.FavoriteMounts.Ground);
            else
                DLU.createMessage(addonData.L.MountUnavailable);
            end
        else
            DLU.createMessage(addonData.L.MountUnavailableInCombat);
        end
    else
        C_MountJournal.Dismiss()
    end
end

-- Helper functions for the UI
local function GetMountType(mountId)
    local _, _, _, _, mountTypeId = C_MountJournal.GetMountInfoExtraByID(mountId);

    if (mountTypeId == 230 or mountTypeId == 284 or mountTypeId == 408 or mountTypeId == 412) then
        return MOUNTTYPES.GROUND;
    elseif (mountTypeId == 231 or mountTypeId == 232 or mountTypeId == 254) then
        -- Swim
    elseif (mountTypeId == 241) then
        -- AQ Bugs
    elseif (mountTypeId == 242 or mountTypeId == 247 or mountTypeId == 248 or mountTypeId == 407) then
        return MOUNTTYPES.FLYING;
    elseif (mountTypeId == 402) then
        return MOUNTTYPES.DRAGONRIDING;
    end
end

local function SetMount(mountType, btn)
    local infoType, info1, info2 = GetCursorInfo();
    --print(infoType, info1, info2);

    if (infoType == "mount") then
        local success = false;

        local name, _, icon, _, _, _, _, _, _, _, _, _, isDragonRiding = C_MountJournal.GetMountInfoByID(info1);
        local mountTypeFlag = GetMountType(info1);
        --local mountId,name,spellId,icon,_,mountType = GetCompanionInfo("mount", info1);

        if (mountType == "flying") then
            if (not isDragonRiding and mountTypeFlag == MOUNTTYPES.FLYING) then
                success = true;
                DLUCharacterSettings.FavoriteMounts.Flying = info1;
            else
                DLU.createMessage(format(addonData.L.InvalidFlying, name));
            end
        elseif (mountType == "ground") then
            if (not isDragonRiding and mountTypeFlag == MOUNTTYPES.GROUND) then
                success = true;
                DLUCharacterSettings.FavoriteMounts.Ground = info1;
            else
                DLU.createMessage(format(addonData.L.InvalidGround, name));
            end
        elseif (mountType == "dragonriding") then
            if (isDragonRiding and mountTypeFlag == MOUNTTYPES.DRAGONRIDING) then
                success = true;
                DLUCharacterSettings.FavoriteMounts.DragonRiding = info1;
            else
                DLU.createMessage(format(addonData.L.InvalidDragonRiding, name));
            end
        end

        if (success) then
            _G[btn:GetName() .. "IconTexture"]:SetTexture(icon);
        end
    else
        DLU.createMessage(addonData.L.InvalidMountItem);
    end
end

-- UI panel
local function CreateMountPanel()
    local width = DLU.GetWidth();
	local height = DLU.GetHeight();
	local padding = DLU.GetPaddings();
	local fonts = DLU.GetFontSizes();

    DLUHQ.Panels.Mounts = DLU.CreatePanel(width.DLUHQ, height.DLUHQ - height.Nav, DLUHQ.Container, -height.Nav);
	local mountHeader = DLU.createTextRoot(addonData.L.MountHeader, "TOP", DLUHQ.Panels.Mounts, 8, -6, "GameFontNormalLarge");

    local groundMountText = DLU.createTextChild("Ground mount:", DLUHQ.Panels.Mounts, mountHeader, -(width.DLUHQ / 2.5), -8, "GameFontHighlightSmall");

    local groundBtn = DLU.createItemSlotButton(DLUHQ.Panels.Mounts, "CENTER");
    groundBtn:SetPoint("TOPLEFT", DLUHQ.Panels.Mounts, "TOPLEFT", (width.DLUHQ / 5.6), -48);
    groundBtn:SetScript("OnReceiveDrag", function()
        SetMount("ground", groundBtn);
    end)

    if (DLUCharacterSettings.FavoriteMounts.Ground ~= nil) then
        local _, _, icon = C_MountJournal.GetMountInfoByID(DLUCharacterSettings.FavoriteMounts.Ground);
        _G[groundBtn:GetName() .. "IconTexture"]:SetTexture(icon);
    end

    local flyingMountText = DLU.createTextChild("Flying mount:", DLUHQ.Panels.Mounts, mountHeader, -(width.DLUHQ / 7.5), -8, "GameFontHighlightSmall");

    local flyingBtn = DLU.createItemSlotButton(DLUHQ.Panels.Mounts, "CENTER");
    flyingBtn:SetPoint("TOPLEFT", DLUHQ.Panels.Mounts, "TOPLEFT", (width.DLUHQ / 2.4), -48);
    flyingBtn:SetScript("OnReceiveDrag", function()
        SetMount("flying", flyingBtn);
    end)

    if (DLUCharacterSettings.FavoriteMounts.Flying ~= nil) then
        local _, _, icon = C_MountJournal.GetMountInfoByID(DLUCharacterSettings.FavoriteMounts.Flying);
        _G[flyingBtn:GetName() .. "IconTexture"]:SetTexture(icon);
    end

    local dragonRidingMountText = DLU.createTextChild("Dragonriding mount:", DLUHQ.Panels.Mounts, mountHeader, (width.DLUHQ / 8), -8, "GameFontHighlightSmall");

    local dragonRidingBtn = DLU.createItemSlotButton(DLUHQ.Panels.Mounts, "CENTER");
    dragonRidingBtn:SetPoint("TOPLEFT", DLUHQ.Panels.Mounts, "TOPLEFT", (width.DLUHQ / 1.35), -48);
    dragonRidingBtn:SetScript("OnReceiveDrag", function()
        SetMount("dragonriding", dragonRidingBtn);
    end)

    if (DLUCharacterSettings.FavoriteMounts.DragonRiding ~= nil) then
        local _, _, icon = C_MountJournal.GetMountInfoByID(DLUCharacterSettings.FavoriteMounts.DragonRiding);
        _G[dragonRidingBtn:GetName() .. "IconTexture"]:SetTexture(icon);
    end
end

-- Event
f:RegisterEvent("PLAYER_LOGIN");
f:SetScript("OnEvent", function()
	CreateMountPanel();

    f:UnregisterEvent("PLAYER_LOGIN");
end);